@@ -8,6 +8,118 @@ class Error(Exception):
 
 	pass
 
+class OperationNotAllowed(Error):
+	'''
+	Exception raised when we try to perform an operation that is not allowed (e.g. write something in read only mode).
+	'''
+
+	pass
+
+class FCollectionCategoryNotFoundError(Error):
+	'''
+	Exception raised when we try to access a non-existing category of an FCollection.
+
+	Parameters
+	----------
+	category : str
+		Name of the category.
+	'''
+
+	def __init__(self, category):
+		self.category = category
+
+class FCollectionFunctionNotFoundError(Error):
+	'''
+	Exception raised when we try to access a non-existing function of an FCollection.
+
+	Parameters
+	----------
+	fname : str
+		Name of the function.
+	'''
+
+	def __init__(self, fname):
+		self.fname = fname
+
+class InvalidFilterRegexError(Error):
+	'''
+	Exception raised when we define the filter regex of an FCollection without the required groups.
+
+	Parameters
+	----------
+	regex : str
+		The invalid regex.
+	'''
+
+	def __init__(self, regex):
+		self.regex = regex
+
+class SettingTagNotRecognizedError(Error):
+	'''
+	Exception raised when we try to analyse a setting tag but it fails.
+
+	Parameters
+	----------
+	setting_tag : str
+		The invalid tag.
+	'''
+
+	def __init__(self, setting_tag):
+		self.setting_tag = setting_tag
+
+class JobAlreadyExistingError(Error):
+	'''
+	Exception raised when we try to add a job to manage that already exists.
+
+	Parameters
+	----------
+	job_name : str
+		Name of the job.
+	'''
+
+	def __init__(self, job_name):
+		self.job_name = job_name
+
+class JobNotFoundError(Error):
+	'''
+	Exception raised when we try to access a non existing job.
+
+	Parameters
+	----------
+	job_name : str
+		Name of the job.
+	'''
+
+	def __init__(self, job_name):
+		self.job_name = job_name
+
+class JobStateNotFoundError(Error):
+	'''
+	Exception raised when we try to read the state of a job from a dict, without the right key.
+	'''
+
+	pass
+
+class UnknownJobStateError(Error):
+	'''
+	Exception raised when we try to use an unknown job state.
+
+	Parameters
+	----------
+	state : str
+		Name of the unknown state.
+	'''
+
+	def __init__(self, state):
+		self.state = state
+
+class ManagerAlreadyRunningError(Error):
+	'''
+	Exception raised when a instance of the Manager is created while another is still running.
+	'''
+
+	pass
+
 class SettingsSetNotFoundError(Error):
 	'''
 	Exception raised when a settings set has not been found.
@@ -107,6 +219,19 @@ class FixerNotFoundError(Error):
 	def __init__(self, fixer_name):
 		self.fixer_name = fixer_name
 
+class NamerNotFoundError(Error):
+	'''
+	Exception raised when we try to access a namer which does not exist.
+
+	Parameters
+	----------
+	namer_name : str
+		Name of the namer which has not been found.
+	'''
+
+	def __init__(self, namer_name):
+		self.namer_name = namer_name
+
 class SimulationIntegrityCheckFailedError(Error):
 	'''
 	Exception raised when a folder fails to pass an integrity check.
@@ -186,32 +311,6 @@ class RemotePathNotFoundError(Error):
 	def __init__(self, remote_path):
 		self.remote_path = remote_path
 
-class UILineNotFoundError(Error):
-	'''
-	Exception raised when we try to access to a UI line which does not exist.
-
-	Parameters
-	----------
-	id : str
-		The ID of the unknown line.
-	'''
-
-	def __init__(self, id):
-		self.id = id
-
-class UIProgressBarNotFoundError(Error):
-	'''
-	Exception raised when we try to access to a UI progress bar which does not exist.
-
-	Parameters
-	----------
-	id : str
-		The ID of the unknown progress bar.
-	'''
-
-	def __init__(self, id):
-		self.id = id
-
 class UINonMovableLine(Error):
 	'''
 	Exception raised when we try to move a line which can't be moved.
@@ -225,44 +324,69 @@ class UINonMovableLine(Error):
 	def __init__(self, pos):
 		self.pos = pos
 
-class WatcherNoConfigFoundError(Error):
+class EventUnknownError(Error):
 	'''
-	Exception raised when nothing is used to configure the watcher.
+	Exception raised when we try to use an unkown event.
+
+	Parameters
+	----------
+	event : str
+		Name of the event.
+	'''
+
+	def __init__(self, event):
+		self.event = event
+
+class MakerPausedError(Error):
+	'''
+	Exception raised when the Maker is paused while it should not be.
 	'''
 
 	pass
 
-class WatcherTooManyConfigError(Error):
+class MakerNotPausedError(Error):
 	'''
-	Exception raised when there is more than one configuration for the watcher.
+	Exception raised when the Maker is not paused while it should be.
 	'''
 
 	pass
 
-class WatcherNoRemoteFolderError(Error):
+class MakerStateWrongFormatError(Error):
 	'''
-	Exception raised when we try to use the watcher's remote folder while there is no one set.
+	Exception raised when we try to read a Maker state that is in the wrong format.
 	'''
 
 	pass
 
-class WatcherNoStatesPathError(Error):
+class ExplorerDepthNotFoundError(Error):
 	'''
-	Exception raised when we try to access the file containing the jobs states and there is no one set.
+	Exception raised when we try to access a non existing depth in a map.
+
+	Parameters
+	----------
+	depth : int
+		The non existing depth.
 	'''
 
-	pass
+	def __init__(self, depth):
+		self.depth = depth
 
-class WatcherNoMailConfigError(Error):
+class ExplorerStopNotFoundError(Error):
 	'''
-	Exception raised when we try to connect to the mailbox and no configuration file is given.
+	Exception raised when we try to access a stop that is not defined.
+
+	Parameters
+	----------
+	depth : int
+		The depth at which the stop was requested.
 	'''
 
-	pass
+	def __init__(self, depth):
+		self.depth = depth
 
-class WatcherNoMailNotificationsConfigError(Error):
+class ExplorerSearchNoSolutionError(Error):
 	'''
-	Exception raised when we try to access the mails notifications configuration without any configuration file provided.
+	Exception raised when we search in the wrong interval.
 	'''
 
 	pass
