@@ -1,12 +1,14 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
+import os
 import copy
 import functools
 import re
+import abc
 
 from .errors import *
-from . import string
+from . import string, jsonfiles
 
 class Simulation():
 	'''
@@ -27,7 +29,8 @@ class Simulation():
 
 		self._raw_globalsettings = None
 		self._raw_settings = None
-		self._raw_settings_dict = None
+
+		self._indexed_settings = None
 
 		self._setting_tag_regex_compiled = None
 		self._eval_tag_regex_compiled = None
@@ -57,6 +60,40 @@ class Simulation():
 
 		return cls(folder, simulation)
 
+	def copy(self):
+		'''
+		Create a copy of the simulation.
+
+		Returns
+		-------
+		simulation : Simulation
+			The copy of the simulation.
+		'''
+
+		return Simulation(self._folder, {
+			**self.globalsettings,
+			'settings': {
+				settings_set_name: [
+					{s.name: s._value for s in settings_set}
+					for settings_set in settings_sets
+				]
+				for settings_set_name, settings_sets in self._settings.items()
+			}
+		})
+
+	@property
+	def folder(self):
+		'''
+		Get the Folder instance used by the simulation.
+
+		Returns
+		-------
+		folder : Folder
+			The Folder instance.
+		'''
+
+		return self._folder
+
 	def __getitem__(self, key):
 		'''
 		Access to a global setting.
@@ -78,7 +115,7 @@ class Simulation():
 		'''
 
 		try:
-			return self.reduced_globalsettings[key]
+			return self.globalsettings[key]
 
 		except KeyError:
 			raise KeyError('The key does not exist in the global settings')
@@ -102,13 +139,28 @@ class Simulation():
 		'''
 
 		try:
-			setting = [setting for setting in self._globalsettings if setting['name'] == key][0]
+			setting = [setting for setting in self._globalsettings if setting.name == key][0]
 
 		except IndexError:
 			raise KeyError('The key does not exist in the global settings')
 
 		else:
-			setting['value'] = value
+			setting.value = value
+
+	def writeSettingsFile(self, filename, *, folder = None):
+		'''
+		Write the settings into a file in the simulation's folder.
+
+		Parameters
+		----------
+		filename : str
+			Name of the file to write.
+
+		folder : str
+			Name of the folder in which the file should be saved. Default to the simulation's folder.
+		'''
+
+		jsonfiles.write(self.settings, os.path.join(folder or self['folder'], filename))
 
 	@property
 	def _globalsettings(self):
@@ -121,29 +173,13 @@ class Simulation():
 			The global settings.
 		'''
 
-		if not(self._raw_globalsettings):
+		if self._raw_globalsettings is None:
 			self.generateGlobalSettings()
 
 		return self._raw_globalsettings
 
 	@property
 	def _settings(self):
-		'''
-		Return (and generate if needed) the complete list of settings.
-
-		Returns
-		-------
-		raw_settings : list
-			The settings.
-		'''
-
-		if not(self._raw_settings):
-			self.generateSettings()
-
-		return self._raw_settings
-
-	@property
-	def _settings_dict(self):
 		'''
 		Return (and generate if needed) the complete list of settings as a dictionary.
 
@@ -153,30 +189,58 @@ class Simulation():
 			The settings.
 		'''
 
-		if not(self._raw_settings_dict):
+		if self._raw_settings is None:
 			self.generateSettings()
 
-		return self._raw_settings_dict
+		return self._raw_settings
 
 	@property
-	def settings(self):
+	def raw_settings(self):
 		'''
-		Return the complete list of sets of settings to use, as dictionaries.
-		The settings with `exclude` to `True` are ignored.
+		Return a dictionary with the complete list of sets of settings.
 
 		Returns
 		-------
-		settings : list
-			List of sets of settings.
+		settings : dict
+			The list of settings.
 		'''
 
-		return [
-			{s.name: s.value for s in settings_set if not(s.exclude)}
-			for settings_set in self._settings
-		]
+		return {
+			settings_set_name: [
+				{s.name: s for s in settings_set}
+				for settings_set in settings_sets
+			]
+			for settings_set_name, settings_sets in self._settings.items()
+		}
+
+	def getSetting(self, coords):
+		'''
+		Retrieve a setting from its set and name.
+
+		Parameters
+		----------
+		coords : dict
+			Describe the "coordinates" of the setting. Must contain the following keys:
+				* `set`: the name of the set the setting belongs to,
+				* `name`: the name of the setting.
+			The following key is optional:
+				* `set_index`: the index of the set. If not provided, default to 0.
+
+		Returns
+		-------
+		setting : SimulationSetting
+			The setting corresponding to the coordinates.
+		'''
+
+		coords = {
+			'set_index': 0,
+			**coords
+		}
+
+		return self.raw_settings[coords['set']][coords['set_index']][coords['name']]
 
 	@property
-	def settings_dict(self):
+	def settings(self):
 		'''
 		Return a dictionary with the complete list of sets of settings to use, as dictionaries.
 		The settings with `exclude` to `True` are ignored.
@@ -192,7 +256,7 @@ class Simulation():
 				{s.name: s.value for s in settings_set if not(s.exclude)}
 				for settings_set in settings_sets
 			]
-			for settings_set_name, settings_sets in self._settings_dict.items()
+			for settings_set_name, settings_sets in self._settings.items()
 		}
 
 	@property
@@ -203,17 +267,20 @@ class Simulation():
 
 		Returns
 		-------
-		settings : list
-			Settings, generated according to their pattern.
+		settings : dict
+			Settings, generated according to their pattern and organized by sets.
 		'''
 
-		return [
-			[str(s) for s in settings_set if s.shouldBeDisplayed()]
-			for settings_set in self._settings
-		]
+		return {
+			set_name: [
+				[str(s) for s in settings_set if s.shouldBeDisplayed()]
+				for settings_set in settings_sets
+			]
+			for set_name, settings_sets in self._settings.items()
+		}
 
 	@property
-	def reduced_globalsettings(self):
+	def globalsettings(self):
 		'''
 		Return the list of global settings, as a name: value dictionary.
 
@@ -223,21 +290,7 @@ class Simulation():
 			The global settings.
 		'''
 
-		return {setting['name']: setting['value'] for setting in self._globalsettings}
-
-	@property
-	def reduced_settings(self):
-		'''
-		Return the list of settings, as a name: value dictionary.
-		Ignore multiple occurrences of the same setting.
-
-		Returns
-		-------
-		settings : dict
-			The settings.
-		'''
-
-		return functools.reduce(lambda a, b: {**a, **b}, self.settings)
+		return {setting.name: setting.value for setting in self._globalsettings}
 
 	@property
 	def command_line(self):
@@ -250,7 +303,7 @@ class Simulation():
 			The command line to execute.
 		'''
 
-		return ' '.join([self._folder.settings['exec']] + sum(self.settings_as_strings, []))
+		return ' '.join([self._folder.settings['exec']] + sum(sum(self.settings_as_strings.values(), []), []))
 
 	@property
 	def _setting_tag_regex(self):
@@ -264,7 +317,7 @@ class Simulation():
 		'''
 
 		if self._setting_tag_regex_compiled is None:
-			self._setting_tag_regex_compiled = re.compile(r'\{(?P<category>(?:global)?setting):(?P<name>[^}]+)\}')
+			self._setting_tag_regex_compiled = re.compile(r'\{(?P<category>(?:global)?setting)(?:\[(?P<setname>.+?)\])?(?:\((?P<index>[0-9]+)\))?:(?P<name>.+?)\}')
 
 		return self._setting_tag_regex_compiled
 
@@ -295,12 +348,114 @@ class Simulation():
 		self._raw_globalsettings = []
 
 		for setting in self._folder.settings['globalsettings']:
-			self._raw_globalsettings.append({
-				'name': setting['name'],
-				'value': self._folder.applyFixers(self._user_settings[setting['name']]) if setting['name'] in self._user_settings else setting['default']
-			})
+			try:
+				setting_value = self._user_settings[setting['name']]
 
-		self.parseGlobalSettings()
+			except KeyError:
+				setting_value = setting['default']
+
+			self._raw_globalsettings.append(SimulationGlobalSetting(self, setting['name'], setting_value))
+
+	def getSettingCount(self, setting_name, set_name = None):
+		'''
+		Get the total number of uses of a given setting, globally or locally.
+
+		Parameters
+		----------
+		setting_name : str
+			Name of the setting to get the count of.
+
+		set_name : str
+			Name of the set for the local count.
+
+		Raises
+		------
+		SettingsSetNotFoundError
+			The provided set name does not exist.
+
+		SettingNotFoundError
+			The provided setting name does not exist.
+
+		Returns
+		-------
+		count : int
+			Local count if `set_name` is not `None`, global count otherwise.
+		'''
+
+		try:
+			settings = self._indexed_settings['global'] if set_name is None else self._indexed_settings['local'][set_name]
+
+		except KeyError:
+			raise SettingsSetNotFoundError(set_name)
+
+		else:
+			try:
+				return len(settings[setting_name])
+
+			except KeyError:
+				raise SettingNotFoundError(set_name, setting_name)
+
+	def _indexSetting(self, setting, set_name):
+		'''
+		Add a setting to the index.
+
+		Parameters
+		----------
+		setting : SimulationSetting
+			The setting to index.
+
+		set_name : str
+			The name of the set the setting belongs to.
+		'''
+
+		if not(set_name in self._indexed_settings['local']):
+			self._indexed_settings['local'][set_name] = {}
+
+		indexes = []
+
+		for indexes_dict in [self._indexed_settings['global'], self._indexed_settings['local'][set_name]]:
+			try:
+				indexes_dict[setting.name].append(setting)
+
+			except KeyError:
+				indexes_dict[setting.name] = [setting]
+
+			indexes.append(len(indexes_dict[setting.name]) - 1)
+
+		setting.setIndexes(*indexes)
+
+	def _addRawSettingsSet(self, set_name, default_settings, values_set = {}):
+		'''
+		Add a raw settings set to the list.
+
+		Parameters
+		----------
+		set_name : str
+			Name of the set.
+
+		default_settings : list
+			List of the default settings of this set.
+
+		values_set : dict
+			Values to override the defaults.
+		'''
+
+		set_to_add = copy.deepcopy(default_settings)
+
+		for setting in set_to_add:
+			self._indexSetting(setting, set_name)
+
+			try:
+				setting.value = values_set[setting.name]
+
+			except KeyError:
+				pass
+
+		try:
+			self._raw_settings[set_name].append(set_to_add)
+
+		except KeyError:
+			self._raw_settings[set_name] = [set_to_add]
 
 	def generateSettings(self):
 		'''
@@ -318,12 +473,14 @@ class Simulation():
 		else:
 			user_settings = self._user_settings['settings']
 
-		self._raw_settings_dict = {}
 		default_pattern = self._folder.settings['setting_pattern']
 
+		self._raw_settings = {}
+		self._indexed_settings = {'global': {}, 'local': {}}
+
 		for settings_set in self._folder.settings['settings']:
 			default_settings = [
-				SimulationSetting(self._folder, settings_set['set'], s['name'], self)
+				SimulationSetting(self, settings_set['set'], s['name'])
 				for s in settings_set['settings']
 			]
 
@@ -332,28 +489,74 @@ class Simulation():
 
 			except KeyError:
 				if settings_set['required']:
-					self._raw_settings_dict[settings_set['set']] = [default_settings]
+					self._addRawSettingsSet(settings_set['set'], default_settings)
 
 			else:
 				if not(type(values_sets) is list):
 					values_sets = [values_sets]
 
-				self._raw_settings_dict[settings_set['set']] = []
-
 				for values_set in values_sets:
-					set_to_add = copy.deepcopy(default_settings)
+					self._addRawSettingsSet(settings_set['set'], default_settings, values_set)
 
-					for setting in set_to_add:
-						try:
-							setting.value = values_set[setting.name]
+	def getSettingValueFromTag(self, match):
+		'''
+		Retrieve the value of a setting from a setting tag.
 
-						except KeyError:
-							pass
+		Parameters
+		----------
+		match : re.Match
+			Match object corresponding to a setting tag.
 
-					self._raw_settings_dict[settings_set['set']].append(set_to_add)
+		Raises
+		------
+		SettingTagNotRecognizedError
+			The setting tag has not been recognized/does not refer to an existing setting.
 
-		self._raw_settings = sum(self._raw_settings_dict.values(), [])
-		self.parseSettings()
+		Returns
+		-------
+		setting : mixed
+			The value of the setting.
+		'''
+
+		try:
+			if match.group('category') == 'globalsetting':
+				return [
+					setting.value
+					for setting in self._globalsettings
+					if setting.name == match.group('name')
+				][0]
+
+			set_dict = self._indexed_settings['global'] if match.group('setname') is None else self._indexed_settings['local'][match.group('setname')]
+			set_list = set_dict[match.group('name')]
+
+			k = 0 if match.group('index') is None else int(match.group('index'))
+
+			return set_list[k].value
+
+		except:
+			raise SettingTagNotRecognizedError(match.group(0))
+
+	def replaceSettingTag(self, match):
+		'''
+		Replace a setting tag (`{setting[set_name](k):setting_name}`) by the value of the right setting.
+		To be called by `re.sub()`.
+
+		Parameters
+		----------
+		match : re.Match
+			Match object corresponding to a setting tag.
+
+		Returns
+		-------
+		setting_value : str
+			The value of the setting.
+		'''
+
+		try:
+			return str(self.getSettingValueFromTag(match))
+
+		except SettingTagNotRecognizedError:
+			return match.group(0)
 
 	def parseString(self, s):
 		'''
@@ -390,35 +593,16 @@ class Simulation():
 
 		# We search for settings tags in the string, and recursively replace them
 
-		settings = {
-			'setting': self.reduced_settings,
-			'globalsetting': self.reduced_globalsettings
-		}
-
 		fullmatch = self._setting_tag_regex.fullmatch(s)
 
 		if fullmatch:
 			try:
-				return copy.deepcopy(settings[fullmatch.group('category')][fullmatch.group('name')])
+				return copy.deepcopy(self.getSettingValueFromTag(fullmatch))
 
 			except KeyError:
 				return s
 
-		parsed = ''
-		k0 = 0
-
-		for match in self._setting_tag_regex.finditer(s):
-			parsed += s[k0:match.start()]
-
-			try:
-				parsed += str(settings[match.group('category')][match.group('name')])
-
-			except KeyError:
-				parsed += match.group(0)
-
-			k0 = match.end()
-
-		parsed += s[k0:]
+		parsed = self._setting_tag_regex.sub(self.replaceSettingTag, s)
 
 		self._parser_recursion_stack.append(s)
 
@@ -444,102 +628,45 @@ class Simulation():
 
 		return parsed
 
-	def parseGlobalSettings(self):
-		'''
-		Parse the global settings to take into account possible other settings' values.
-		'''
-
-		for setting in self._globalsettings:
-			setting['value'] = self.parseString(setting['value'])
-
-	def parseSettings(self):
-		'''
-		Parse the settings to take into account possible other settings' values.
-		'''
-
-		for settings_set in self._settings:
-			for setting in settings_set:
-				setting.value = self.parseString(setting.value)
-
-class SimulationSetting():
+class SimulationBaseSetting(abc.ABC):
 	'''
-	Represent a simulation setting.
+	Represent a setting, either global or normal (abstract class).
 
 	Parameters
 	----------
-	folder : Folder
-		The simulations folder this setting is part of.
-
-	set_name : str
-		Name of the set this setting is part of.
+	simulation : Simulation
+		The simulation using this setting.
 
 	setting_name : str
-		Name of this setting.
+		Name of the setting.
 
-	simulation : Simulation
-		Simulation that uses this setting.
-
-	Raises
-	------
-	SettingsSetNotFoundError
-		The settings set has not been found.
-
-	SettingNotFoundError
-		The setting has not been found in the set.
+	setting_value : mixed
+		Default value of the setting.
 	'''
 
-	def __init__(self, folder, set_name, setting_name, simulation = None):
-		self._folder = folder
+	def __init__(self, simulation, setting_name, setting_value):
 		self._simulation = simulation
 
-		try:
-			settings_set_dict = [
-				settings_set
-				for settings_set in self._folder.settings['settings']
-				if settings_set['set'] == set_name
-			][0]
+		self._name = setting_name
+		self._value = setting_value
 
-		except IndexError:
-			raise SettingsSetNotFoundError(set_name)
-
-		try:
-			setting_dict = [
-				setting
-				for setting in settings_set_dict['settings']
-				if setting['name'] == setting_name
-			][0]
-
-		except IndexError:
-			raise SettingNotFoundError(set_name, setting_name)
-
-		self._name = setting_dict['name']
-		self._value = setting_dict['default']
-		self._exclude_for_db = 'exclude' in setting_dict and setting_dict['exclude']
-		self._pattern = setting_dict['pattern'] if 'pattern' in setting_dict else self._folder.settings['setting_pattern']
-
-		self._use_only_if = 'only_if' in setting_dict
-		if self._use_only_if:
-			self._only_if_value = setting_dict['only_if']
-
-		self._fixers = {'before': [], 'after': []}
-		for when in ['before', 'after']:
-			try:
-				self._fixers[when] = setting_dict[f'fixes_{when}']
-
-			except KeyError:
-				pass
-
-	def __str__(self):
+	def __deepcopy__(self, memo):
 		'''
-		Return a string representing the setting and its value, according to the pattern.
-
-		Returns
-		-------
-		setting : str
-			The representation of the setting.
+		Override the default behavior of `deepcopy()` to keep the references to the Simulation.
 		'''
 
-		return self._pattern.format(name = self.name, value = self.value)
+		cls = self.__class__
+		result = cls.__new__(cls)
+		memo[id(self)] = result
+
+		for k, v in self.__dict__.items():
+			if k == '_simulation':
+				setattr(result, k, v)
+
+			else:
+				setattr(result, k, copy.deepcopy(v, memo))
+
+		return result
 
 	@property
 	def name(self):
@@ -557,7 +684,7 @@ class SimulationSetting():
 	@property
 	def value(self):
 		'''
-		Getter for the setting value.
+		Getter for the setting value, with fixers applied.
 
 		Returns
 		-------
@@ -565,7 +692,13 @@ class SimulationSetting():
 			Value of the setting.
 		'''
 
-		return self._value
+		try:
+			fixers = self._fixers
+
+		except AttributeError:
+			fixers = {}
+
+		return self._simulation.folder.applyFixers(self._simulation.parseString(self._value), **fixers)
 
 	@value.setter
 	def value(self, new_value):
@@ -578,7 +711,190 @@ class SimulationSetting():
 			New value of the setting
 		'''
 
-		self._value = self._folder.applyFixers(new_value, **self._fixers)
+		self._value = new_value
+
+class SimulationGlobalSetting(SimulationBaseSetting):
+	'''
+	Represent a global setting.
+	'''
+
+	pass
+
+class SimulationSetting(SimulationBaseSetting):
+	'''
+	Represent a simulation setting.
+
+	Parameters
+	----------
+	simulation : Simulation
+		Simulation that uses this setting.
+
+	set_name : str
+		Name of the set this setting is part of.
+
+	setting_name : str
+		Name of this setting.
+
+	Raises
+	------
+	SettingsSetNotFoundError
+		The settings set has not been found.
+
+	SettingNotFoundError
+		The setting has not been found in the set.
+	'''
+
+	def __init__(self, simulation, set_name, setting_name):
+		try:
+			self._settings_set_dict = [
+				settings_set
+				for settings_set in simulation.folder.settings['settings']
+				if settings_set['set'] == set_name
+			][0]
+
+		except IndexError:
+			raise SettingsSetNotFoundError(set_name)
+
+		try:
+			self._setting_dict = [
+				setting
+				for setting in self._settings_set_dict['settings']
+				if setting['name'] == setting_name
+			][0]
+
+		except IndexError:
+			raise SettingNotFoundError(set_name, setting_name)
+
+		super().__init__(simulation, setting_name, self._setting_dict['default'])
+
+		self._set_name = set_name
+		self._exclude_for_db = 'exclude' in self._setting_dict and self._setting_dict['exclude']
+		self._pattern = self._setting_dict['pattern'] if 'pattern' in self._setting_dict else self._simulation.folder.settings['setting_pattern']
+
+		self._use_only_if = 'only_if' in self._setting_dict
+		if self._use_only_if:
+			self._only_if_value = self._setting_dict['only_if']
+
+		self._fixers_dict = None
+		self._namers_dict = None
+
+	def __str__(self):
+		'''
+		Return a string representing the setting and its value, according to the pattern.
+
+		Returns
+		-------
+		setting : str
+			The representation of the setting.
+		'''
+
+		return self._pattern.format(name = self.display_name, value = self.value)
+
+	def setIndexes(self, global_index, local_index):
+		'''
+		Define the global and local indexes of this setting.
+
+		Parameters
+		----------
+		global_index : int
+			Global index of this setting.
+
+		local_index : int
+			Local index of this setting.
+		'''
+
+		self._global_index = global_index
+		self._local_index = local_index
+
+	def _setModifier(self, modifier):
+		'''
+		Set a "modifier" property (fixers or namers).
+
+		Parameters
+		----------
+		modifier : str
+			Modifier to define.
+
+		Returns
+		-------
+		modifier_functions : dict
+			The functions to call, in the right order.
+		'''
+
+		modifier_functions = {'before': [], 'after': []}
+
+		keys_to_search = {
+			'before': [
+				(self._setting_dict, ''),
+				(self._settings_set_dict, ''),
+				(self._setting_dict, '_before'),
+				(self._settings_set_dict, '_before'),
+				(self._setting_dict, '_between_before')
+			],
+			'after': [
+				(self._setting_dict, '_between_after'),
+				(self._settings_set_dict, '_after'),
+				(self._setting_dict, '_after')
+			]
+		}
+
+		for when, keys_path in keys_to_search.items():
+			for dict_to_search, key_to_search in keys_path:
+				try:
+					modifier_functions[when] += dict_to_search[f'{modifier}{key_to_search}']
+
+				except KeyError:
+					pass
+
+		return modifier_functions
+
+	@property
+	def _fixers(self):
+		'''
+		Get the fixers to apply to the value.
+
+		Returns
+		-------
+		fixers : dict
+			The list of fixers, in the right order.
+		'''
+
+		if self._fixers_dict is None:
+			self._fixers_dict = self._setModifier('fixers')
+
+		return self._fixers_dict
+
+	@property
+	def _namers(self):
+		'''
+		Get the namers to apply to the name.
+
+		Returns
+		-------
+		namers : dict
+			The list of namers, in the right order.
+		'''
+
+		if self._namers_dict is None:
+			self._namers_dict = self._setModifier('namers')
+
+		return self._namers_dict
+
+	@property
+	def display_name(self):
+		'''
+		Get the name of the setting to use inside the simulation.
+
+		Returns
+		-------
+		name : str
+			Name to use.
+		'''
+
+		global_total = self._simulation.getSettingCount(self.name)
+		local_total = self._simulation.getSettingCount(self.name, self._set_name)
+
+		return self._simulation.folder.applyNamers(self.name, self._local_index, local_total, self._global_index, global_total, **self._namers)
 
 	@property
 	def exclude(self):
